﻿Param
(
    [Parameter(Mandatory=$false)]
    [string]$Ensure = 'Present',          #specify if you want to manage custom admin account rather than built-in one

    [Parameter(Mandatory=$false)]
    [string]$AdminAccountName,          #specify if you want to manage custom admin account rather than built-in one

	[Parameter(Mandatory=$false)]
    [Boolean]$Enabled=$true,            #solution 'master switch' - client does not do anything unless this is set to $true

	[Parameter(Mandatory=$false)]
    [UInt32]$PasswordLength=14,         #chars in password

	[Parameter(Mandatory=$false)]
    [UInt32]$PasswordComplexity=3,      #0=Large, 1=LargeSmall ,2=LargeSmallNum, 3=LargeSmallNumSpec

	[Parameter(Mandatory=$false)]
    [UInt32]$PasswordAge=720,            #hours (=30 days)

	[Parameter(Mandatory=$false)]
    [UInt32]$ManagementInterval=3600,   #seconds (60 mins); how often management cycle occurs on client

	[Parameter(Mandatory=$false)]
	[Boolean]$PasswordEncryptionEnabled = $false,    #password encryption switch

    [Parameter(Mandatory=$false)]
    [string]$PasswordEncryptionKey,          #encryption key. Public key as produced by PDS. Only used when PasswordfEncryptionEnabled=$true

    [Parameter(Mandatory=$false)]
	[Boolean]$PasswordManualChangeProtectionEnabled = $true,    #protect against manual password changes. Client resets the passwordf if it finds out that password was changed by someone

    [Parameter(Mandatory=$false)]
	[Boolean]$PasswordExpirationProtectionEnabled = $false,    #protect against manual reset of password behind max password age

    [Parameter(Mandatory=$false)]
	[Boolean]$PasswordHistoryEnabled = $false,    #maintain password history

	[Parameter(Mandatory=$false)]
    [UInt32]$LogLevel=0                 #0=Error, 1=ErrorsWarnings, 2=All
)

$ConfigData = @{
   AllNodes = @(
      @{ NodeName = "*" }
        )
}
 
Configuration AdmPwd_E_Config {
	Param (
 
		[Parameter()]
		[ValidateSet("Present","Absent")]
		[String]$Ensure = $Ensure
	)
	
	#where configfuration data is stored in registry
	$Key = 'HKEY_LOCAL_MACHINE\SOFTWARE\Policies\Microsoft Services\AdmPwd'
	
	#required resources for confifguration - we use Registry resource
	Import-DscResource –ModuleName 'PSDesiredStateConfiguration'
 
	if($AdminAccountName -ne $null)
	{
		Registry AdminAccountName
		{
			Ensure = $Ensure
			Key = $Key
			ValueName = 'AdminAccountName'
			ValueType = 'String'
			ValueData = $AdminAccountName
		}
	}
	else
	{
		Registry AdminAccountName
		{
			Ensure = "Absent"
			Key = $Key
			ValueName = 'AdminAccountName'
			ValueType = 'String'
		}
	}

	if($PasswordEncryptionKey -ne $null)
	{
		Registry EncryptionKey
		{
			Ensure = $Ensure
			Key = $Key
			ValueName = 'PublicKey'
			ValueType = 'String'
			ValueData = $PasswordEncryptionKey
		}
	}
	else
	{
		Registry EncryptionKey
		{
			Ensure = "Absent"
			Key = $Key
			ValueName = 'PublicKey'
			ValueType = 'String'
		}
	}

   
	Registry Enabled
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'AdmPwdEnabled'
		ValueType = 'DWord'
		ValueData = ($Enabled -as [Int32])
	}
 
	Registry PasswordLength
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'PasswordLength'
		ValueType = 'DWord'
		ValueData = $PasswordLength
	}
 
	Registry PasswordComplexity
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'PasswordComplexity'
		ValueType = 'DWord'
		ValueData = $PasswordComplexity
	}
   
	Registry PasswordAge
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'PasswordAge'
		ValueType = 'DWord'
		ValueData = $PasswordAge
	}
   
	Registry ManagementInterval
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'PwdManagementInterval'
		ValueType = 'DWord'
		ValueData = $ManagementInterval
	}
   
	Registry ManualChangeProtection
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'ManualPasswordChangeProtectionEnabled'
		ValueType = 'DWord'
		ValueData = ($PasswordManualChangeProtectionEnabled -as [Int32])
	}

	Registry ExpirationProtection
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'PwdExpirationProtectionEnabled'
		ValueType = 'DWord'
		ValueData = ($PasswordExpirationProtectionEnabled -as [Int32])
	}

	Registry PasswordHistory
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'PwdHistoryEnabled'
		ValueType = 'DWord'
		ValueData = ($PasswordHistoryEnabled -as [Int32])
	}

	Registry PasswordEncryption
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'PwdEncryptionEnabled'
		ValueType = 'DWord'
		ValueData =( $PasswordEncryptionEnabled -as [Int32])
	}
	
	Registry LogLevel
	{
		Ensure = $Ensure
		Key = $Key
		ValueName = 'LogLevel'
		ValueType = 'DWord'
		ValueData = $LogLevel
	}
}

AdmPwd_E_Config -ConfigurationData $ConfigData -Ensure Present
Start-DscConfiguration -Wait -Force -Path .\AdmPwd_E_Config -Verbose